﻿

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False

    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function


    ' =======================================================================================================
    '   Platform selection
    ' =======================================================================================================
    Friend Function CanUseWindowsDlls() As Boolean
        Select Case Environment.OSVersion.Platform
            Case PlatformID.Win32NT, PlatformID.Win32S, PlatformID.Win32Windows, PlatformID.WinCE
                Return True
            Case Else
                Return False
        End Select
    End Function
    Friend Function PlatformAdjustedFileName(ByVal FileName As String, _
                                             Optional ByVal DefaultName As String = "") As String
        If CanUseWindowsDlls() Then
            FileName = Replace(FileName, "/", "\")
            Return FileName
        Else
            If FileName.Contains(":") Then FileName = DefaultName
            FileName = Replace(FileName, "\", "/")
            Return FileName
        End If
    End Function


    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 100 - .Width) ' at least at left border + 50 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Combo functions
    ' ================================================================================================
    Friend Sub Combo_InitFromEnum(ByVal combo As ComboBox, ByVal enumType As Type) ' use GeType(enum)
        For Each s As String In [Enum].GetNames(enumType)
            combo.Items.Add(s)
        Next
    End Sub


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                               Optional ByVal Value As Double = Double.NaN, _
                               Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt, GCI)
        End If
    End Function
    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function
    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function
    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function
    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function
    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function
    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function
    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub Save_INI()
        '
        Dim appname As String = IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath)
        Dim iniFileName As String = Application.StartupPath + "\" + appname + "_INI.txt"
        '
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            ' ------------------------------------------------------------------------------ FORM BOUNDS
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            f.WriteLine(TabString("Form1_Width", r.Width))
            f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            '
            f.WriteLine(TabString(""))
            f.WriteLine(TabString(" Slots"))
            f.WriteLine("===========================================")
            f.WriteLine(TabString("SlotOutput", Form1.txt_SlotOutput.NumericValueInteger))
            f.WriteLine(TabString("SlotAmplitude", Form1.txt_SlotAmplitude.NumericValueInteger))
            f.WriteLine(TabString("SlotShift", Form1.txt_SlotShift.NumericValueInteger))
            f.WriteLine(TabString("SlotFrequency", Form1.txt_SlotFrequency.NumericValueInteger))
            f.WriteLine(TabString("SlotWaveform", Form1.txt_SlotWaveform.NumericValueInteger))
            '
            f.WriteLine(TabString(""))
            f.WriteLine(TabString(" Controls"))
            f.WriteLine("===========================================")
            f.WriteLine(TabString("Amplitude", Form1.txt_Amplitude.NumericValue))
            f.WriteLine(TabString("Shift", Form1.txt_Shift.NumericValue))
            f.WriteLine(TabString("Frequency", Form1.txt_Frequency.NumericValue))
            f.WriteLine(TabString("ConstantSpeed", Form1.chk_ConstantSpeed.Checked))
            f.WriteLine(TabString("MinFreq", Form1.txt_MinFreq.NumericValue))
            f.WriteLine(TabString("Shape", Form1.cmb_Shape.Text))
            f.WriteLine(TabString("RunEnabled", RunEnabled))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub


    Friend Sub Load_INI()
        ' ------------------------------------------------------------- defaults
        '
        ' -------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        '
        Dim appname As String = IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath)
        Dim iniFileName As String = Application.StartupPath + "\" + appname + "_INI.txt"
        '
        Dim l As String
        If My.Computer.FileSystem.FileExists(iniFileName) Then

            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Form1_Top" : Form1.Top = Val_Int(l)
                    Case "Form1_Left" : Form1.Left = Val_Int(l)
                        'Case "Form1_Width" : Form1.Width = Val_Int(l)
                        'Case "Form1_Height" : Form1.Height = Val_Int(l)
                    Case "Form1_WindowState" : Form1.WindowState = CType((Val(l)), FormWindowState)
                        ' --------------------------------------------------------------- Params
                    Case "SlotOutput" : Form1.txt_SlotOutput.NumericValueInteger = Val_Int(l)
                    Case "SlotAmplitude" : Form1.txt_SlotAmplitude.NumericValueInteger = Val_Int(l)
                    Case "SlotShift" : Form1.txt_SlotShift.NumericValueInteger = Val_Int(l)
                    Case "SlotFrequency" : Form1.txt_SlotFrequency.NumericValueInteger = Val_Int(l)
                    Case "SlotWaveform" : Form1.txt_SlotWaveform.NumericValueInteger = Val_Int(l)
                        ' --------------------------------------------------------------- Params
                    Case "Amplitude" : Form1.txt_Amplitude.NumericValue = Val_Double(l)
                    Case "Shift" : Form1.txt_Shift.NumericValue = Val_Double(l)
                    Case "Frequency" : Form1.txt_Frequency.NumericValue = Val_Double(l)
                    Case "ConstantSpeed" : Form1.chk_ConstantSpeed.Checked = l = "True"
                    Case "MinFreq" : Form1.txt_MinFreq.NumericValue = Val_Double(l)
                    Case "Shape" : Form1.cmb_Shape.Text = l
                    Case "RunEnabled" : RunEnabled = l = "True"
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
    End Sub


    Friend EcgData1() As Single
    Friend EcgData2() As Single
    Friend OcgData1() As Single
    Friend OcgData2() As Single
    Friend OcgData3() As Single

    Friend UserShape1() As Single
    Friend UserShape2() As Single
    Friend UserShape3() As Single
    Friend UserShape4() As Single
    Friend UserShape5() As Single

    Friend Sub Load_ECG_OCG_Data()
        Load_ECG_Data(My.Resources.ECG_1, EcgData1)
        Load_ECG_Data(My.Resources.ECG_2, EcgData2)
        Load_ECG_Data(My.Resources.OCG_1, OcgData1)
        Load_ECG_Data(My.Resources.OCG_2, OcgData2)
        Load_ECG_Data(My.Resources.OCG_3, OcgData3)

        Load_UserShape("Shape1.txt", UserShape1)
        Load_UserShape("Shape2.txt", UserShape2)
        Load_UserShape("Shape3.txt", UserShape3)
        Load_UserShape("Shape4.txt", UserShape4)
        Load_UserShape("Shape5.txt", UserShape5)
    End Sub

    Friend Sub Load_ECG_Data(ByVal ResourceString As String, ByRef array As Single())
        Dim s As String()
        s = ResourceString.Split(Chr(13))
        ReDim array(s.Length - 1)
        For i As Int32 = 0 To s.Length - 1
            array(i) = CSng(Val(s(i)))
        Next
        ' ---------------------------------------------
        Array_Resample(array, 4000)
    End Sub

    Friend Sub Load_UserShape(ByVal fname As String, ByRef array As Single())
        '
        fname = Application.StartupPath + "\UserShapes\" + fname
        '
        If My.Computer.FileSystem.FileExists(fname) Then
            ' ---------------------------------------------
            Dim reader As System.IO.TextReader
            reader = New System.IO.StreamReader(fname)
            Dim text As String = reader.ReadToEnd
            reader.Close()
            ' ---------------------------------------------
            Dim s As String()
            s = text.Split(Chr(13))
            ReDim array(s.Length - 1)
            For i As Int32 = 0 To s.Length - 1
                array(i) = CSng(Val(s(i)))
            Next
        End If
        ' ---------------------------------------------
        If array Is Nothing OrElse array.Length < 2 Then
            ReDim array(1)
            array(0) = 0
            array(1) = 1000
        End If
    End Sub

End Module
