﻿Module Module_WaveGenerator

    Friend WithEvents AccTimer As AccurateTimer = New AccurateTimer()

    Friend Enum WaveShapes As Int32
        Sinusoidal
        Semi_sinusoidal
        Triangular
        Sawtooth
        Square
        Noise
        Ecg1
        Ecg2
        Ocg1
        Ocg2
        Ocg3
        UserShape1
        UserShape2
        UserShape3
        UserShape4
        UserShape5
    End Enum
    Friend WaveShape As WaveShapes
    '
    Friend SlotOut As Int32
    Friend SlotAmp As Int32
    Friend SlotShift As Int32
    Friend SlotFreq As Int32
    Friend SlotWaveform As Int32
    '
    Friend Amplitude As Double = 1000
    Friend Shift As Double = 500
    Friend ConstantSpeed As Boolean = False
    Friend MinFreq As Single = 210

    Friend NewValue As Double
    Private Const kangle As Single = 2 * Math.PI / 1000.0F

    Friend RunEnabled As Boolean
    Friend Frequency As Double
    Friend NewFrequency As Double
    Private phase As Double
    Private OldMillisec As Double
    Private t1 As Double
    Private t2 As Double
    Private newt As Double

    Private m_randomInt As Int32 = 1
    Private m_randomFloat As Single

    ' ====================================================================================================
    '   Auto Changer
    ' ====================================================================================================
    Friend Sub AccTimer_TimerTick(ByVal milliseconds As Double) Handles AccTimer.Tick
        ' --------------------------------------------------------- slot amplitude
        If SlotAmp >= 0 Then
            Amplitude = Slots.ReadSlot_NoNan(SlotAmp)
        End If
        ' --------------------------------------------------------- slot shift
        If SlotShift >= 0 Then
            Shift = Slots.ReadSlot_NoNan(SlotShift)
        End If
        '
        If Not RunEnabled Then Return
        '
        ' --------------------------------------------------------- slot frequency
        If SlotFreq >= 0 Then
            NewFrequency = Slots.ReadSlot_NoNan(SlotFreq)
            If NewFrequency < 0.01 Then NewFrequency = 0.01
        End If
        ' --------------------------------------------------------- smooth frequency change
        If NewFrequency <> Frequency Then
            newt = (OldMillisec * NewFrequency) Mod 1000 + phase
            phase -= newt - t1
            If phase < 0 Then phase = (phase + 100000) Mod 1000
            Frequency = NewFrequency
        End If
        ' --------------------------------------------------------- prepare values
        OldMillisec = milliseconds
        t1 = (milliseconds * Frequency) Mod 1000 + phase
        t2 = t1 Mod 1000
        ' --------------------------------------------------------- generate wave
        Select Case WaveShape

            Case WaveShapes.Sinusoidal
                NewValue = Math.Sin(t2 * kangle) * 500 + 500

            Case WaveShapes.Semi_sinusoidal
                NewValue = Math.Abs(Math.Sin(t2 * kangle)) * 1000

            Case WaveShapes.Triangular
                If t2 >= 500 Then
                    NewValue = (1000 - t2) * 2
                Else
                    NewValue = t2 * 2
                End If

            Case WaveShapes.Sawtooth
                NewValue = t2

            Case WaveShapes.Square
                If t2 >= 500 Then
                    NewValue = 1000
                Else
                    NewValue = 0
                End If

            Case WaveShapes.Noise
                ' --------------------------------------------------------- not used because Dim = 1
                'If m_randomInt = 0 Then m_randomInt = 1
                ' --------------------------------------------------------- DAA random method
                If (m_randomInt And &H800000) <> 0 Then
                    m_randomInt = (m_randomInt << 1) Xor &H1D872B41
                Else
                    m_randomInt = m_randomInt << 1
                End If
                ' --------------------------------------------------------- limit randomFloat value
                m_randomFloat += m_randomInt * 0.0002F
                If Math.Abs(m_randomFloat) > 1000000000.0 Then m_randomFloat = 0
                m_randomFloat = CInt(m_randomFloat) And 32767
                ' --------------------------------------------------------- set the value
                NewValue = m_randomFloat / 32.768F

            Case WaveShapes.Ecg1
                t2 = t2 / 1000 * EcgData1.Length
                NewValue = EcgData1(CInt(Math.Floor(t2 Mod (EcgData1.Length))))
            Case WaveShapes.Ecg2
                t2 = t2 / 1000 * EcgData2.Length
                NewValue = EcgData2(CInt(Math.Floor(t2 Mod (EcgData2.Length))))
            Case WaveShapes.Ocg1
                t2 = t2 / 1000 * OcgData1.Length
                NewValue = OcgData1(CInt(Math.Floor(t2 Mod (OcgData1.Length))))
            Case WaveShapes.Ocg2
                t2 = t2 / 1000 * OcgData2.Length
                NewValue = OcgData2(CInt(Math.Floor(t2 Mod (OcgData2.Length))))
            Case WaveShapes.Ocg3
                t2 = t2 / 1000 * OcgData3.Length
                NewValue = OcgData3(CInt(Math.Floor(t2 Mod (OcgData3.Length))))

            Case WaveShapes.UserShape1
                t2 = t2 / 1000 * UserShape1.Length
                NewValue = UserShape1(CInt(Math.Floor(t2 Mod (UserShape1.Length))))
            Case WaveShapes.UserShape2
                t2 = t2 / 1000 * UserShape2.Length
                NewValue = UserShape2(CInt(Math.Floor(t2 Mod (UserShape2.Length))))
            Case WaveShapes.UserShape3
                t2 = t2 / 1000 * UserShape3.Length
                NewValue = UserShape3(CInt(Math.Floor(t2 Mod (UserShape3.Length))))
            Case WaveShapes.UserShape4
                t2 = t2 / 1000 * UserShape4.Length
                NewValue = UserShape4(CInt(Math.Floor(t2 Mod (UserShape4.Length))))
            Case WaveShapes.UserShape5
                t2 = t2 / 1000 * UserShape5.Length
                NewValue = UserShape5(CInt(Math.Floor(t2 Mod (UserShape5.Length))))

        End Select

        ' --------------------------------------------------------- constant speed
        If ConstantSpeed Then
            NewValue = (NewValue - 500.0F) * MinFreq / Frequency + 500
        End If
        ' --------------------------------------------------------- write value to slot
        WriteValueToSlot(NewValue)

    End Sub

    Friend Sub WriteValueToSlot(ByVal v As Double)
        v -= 500
        v = v * Amplitude / 1000.0F
        v += Shift
        Slots.WriteSlot(SlotOut, CSng(v))
    End Sub

End Module
